﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion
using System.Collections.Generic;
using HeuristicLab.Core;
using System;
using HeuristicLab.Common;
namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  public interface ISymbolicExpressionTreeNode : IDeepCloneable {
    ISymbolicExpressionTreeGrammar Grammar { get; }
    ISymbolicExpressionTreeNode Parent { get; set; }
    ISymbol Symbol { get; }
    bool HasLocalParameters { get; }
    IEnumerable<ISymbol> GetAllowedSymbols(int argumentIndex);

    int GetHeight();
    int GetSize();
    int GetMinSubtreeCount();
    int GetMaxSubtreeCount();

    IEnumerable<ISymbolicExpressionTreeNode> IterateNodesPostfix();
    IEnumerable<ISymbolicExpressionTreeNode> IterateNodesPrefix();
    void ForEachNodePostfix(Action<ISymbolicExpressionTreeNode> a);
    void ForEachNodePrefix(Action<ISymbolicExpressionTreeNode> a);

    IEnumerable<ISymbolicExpressionTreeNode> SubTrees { get; }
    ISymbolicExpressionTreeNode GetSubTree(int index);
    int IndexOfSubTree(ISymbolicExpressionTreeNode tree);
    void AddSubTree(ISymbolicExpressionTreeNode tree);
    void InsertSubTree(int index, ISymbolicExpressionTreeNode tree);
    void RemoveSubTree(int index);

    void ResetLocalParameters(IRandom random);
    void ShakeLocalParameters(IRandom random, double shakingFactor);
  }
}
